from parroter.Library.parroterConfig import parroterConfig

class kojimaConfig(parroterConfig):
    
    """
    児嶋共通設定クラス(kojimaConfig)
    """ 
    
    class wise_saying_pattern():
        
        """
        名言パターンリスト
        """
        
        def __init__(self):
            
            """
            コンストラクタ
            """
            
            self.pattern = ''
            self.font_name = ''
            self.say_ipX = 0
            self.say_ipY = 0
            self.say_fontSize = 0
            self.say_indentYPoint = 0
            self.say_txtHeight = 0
            self.category_ipX = 0
            self.category_ipY = 0
            self.category_fontSize = 0
            self.category_indentYPoint = 0
            self.category_txtHeight = 0
            self.speaker_ipX = 0
            self.speaker_ipY = 0
            self.speaker_fontSize = 0
            self.speaker_indentYPoint = 0
            self.speaker_txtHeight = 0
            self.category_label = ''

    def __init__(self):
        
        """
        コンストラクタ
        """
        
        # 基底側コンストラクタ
        super().__init__()

        self.wiseSayingJsonDir = ''
        """ 名言json格納先ディレクトリ """

        self.wiseSayingTweetNoLogDir = ''
        """ 名言TweetNoログ格納先ディレクトリ """

        self.wiseSayingJsonFileName = ''
        """ 名言jsonファイル名 """

        self.wiseSayingPtrnJsonFileName = ''
        """ 名言パターンリストjsonファイル名 """

        self.wiseSayingTweetNoLogFileName = ''
        """ 名言TweetNoログファイル名 """

        self.wiseSayingPattern = self.wise_saying_pattern()
        """ 名言パターンリスト """

        self.replyTweetNoLogFileName = ''
        """ 返信TweetNoログファイル名 """

        self.lastnameTargetStrTextFileName = ''
        """ 苗字判定用テキストファイル名 """
        
    def getConfigFileName(self):
        
        """ 
        設定ファイル名 
        """

        return 'kojima.ini'
    
    def setInstanceMemberValues(self):
        
        """ 
        インスタンス変数に読み取った設定値をセットする
        """
        
        # 基底側実行
        super().setInstanceMemberValues()

        # wise_saying - json - FileName
        super().setConfigValue('wiseSayingJsonDir',self.config_ini,'DIR','WISE_SAYING_JSON_DIR',str)

        # wise_saying - json - FileName
        super().setConfigValue('wiseSayingTweetNoLogDir',self.config_ini,'DIR','WISE_SAYING_LOG_DIR',str)
        
        # wise_saying - json - FileName
        super().setConfigValue('wiseSayingJsonFileName',self.config_ini,'FILE','WISE_SAYING_JSON_FILENAME',str)

        # wise_saying_pattern - json - FileName
        super().setConfigValue('wiseSayingPtrnJsonFileName',self.config_ini,'FILE','WISE_SAYING_PATTERN_JSON_FILENAME',str)

        # wise_saying - log - FileName
        super().setConfigValue('wiseSayingTweetNoLogFileName',self.config_ini,'FILE','WISE_SAYING_LOG_FILENAME',str)

        # reply - log - FileName
        super().setConfigValue('replyTweetNoLogFileName',self.config_ini,'FILE','REPLY_LOG_FILENAME',str)

        # lastname_targetStr_txt-FileName
        super().setConfigValue('lastnameTargetStrTextFileName',self.config_ini,'FILE', 'LAST_NAME_TARGET_STR_TXT_FILENAME',str)
        


from Scrapinger.Library.browserContainer import browserContainer
from Scrapinger.Library.webDriverController import webDriverController
from parroter.Library.kojima.kojimaGlobals  import *

class lastNameListController(browserContainer):
    
    """
    苗字リスト取得コントローラー
    """
    
    def __init__(self) -> None:
        
        """
        コンストラクタ
        """
        
        # 基底側コンストラクタ
        super().__init__()
                
        # WebDriverController
        self.wdc = webDriverController(gv.kojimaConfig, self)
        


from LibHanger.Library.uwGlobals import globalValues
from parroter.Library.kojima.kojimaConfig import kojimaConfig

class parroterGlobal(globalValues):
    
    """
    parroterグローバル設定クラス
    """
    
    def __init__(self):
        
        """
        コンストラクタ
        """
        
        # 基底側コンストラクタ呼び出し
        super().__init__()
        


import re
import pandas as pd
import urllib.parse as urlParse
import LibHanger.Library.uwLogger as Logger
from pandas.core.frame import DataFrame
from bs4 import BeautifulSoup
from LibHanger.Models.recset import recset
from Scrapinger.Library.browserContainer import browserContainer
from sqlalchemy.sql.elements import Null
from parroter.Library.kojima.lastNameListController import lastNameListController
from parroter.Library.kojima.kojimaConfig import kojimaConfig
from parroter.Library.kojima.kojimaGlobals import *
from parroter.Models.trn_last_name import trn_last_name

class gettr_lastNameList(lastNameListController):
    
    """
    全国苗字ランキング取得クラス
    """
    
    def __init__(self) -> None:
        
        """
        コンストラクタ
        """
        
        super().__init__()
        
        self.dfLastNameInfo:DataFrame = recset(trn_last_name).getDataFrame()
        """ 全国苗字情報DataFrame """  
    
    def getAllData(self, *args, **kwargs):
        
        for page in range(kwargs.get('FromPage') , int(kwargs.get('ToPage')) + 1, 1):
            dfLastNameInfoTemp = self.getData(pageNo=page)
            if len(dfLastNameInfoTemp) == 0:
                continue
            else:
                self.dfLastNameInfo = pd.concat([self.dfLastNameInfo, dfLastNameInfoTemp])
                
        return self.dfLastNameInfo
    
    @Logger.loggerDecorator("getData",['pageNo'])
    def getData(self, *args, **kwargs):
        
        """
        全国苗字情報取得
        
        Parameters
        ----------
        kwargs : dict
            @pageNo
                ページ番号
        """
        
        # 全国苗字情報をDataFrameで取得
        dfLastNameInfo = self.getLastNameInfoDataFrame(**kwargs)

        return dfLastNameInfo
    
    @Logger.loggerDecorator("getLastNameInfoDataFrame")
    def getLastNameInfoDataFrame(self, *args, **kwargs):

        """
        全国苗字情報取得
        
        Parameters
        ----------
        kwargs : dict
            @pageNo
                ページ番号
        """
        
        # 検索url(ルート)
        rootUrl = gv.kojimaConfig.myoji_yurai_URL
        # 検索url(ページ番号指定)
        lastNameInfoUrl = rootUrl.format(kwargs.get('pageNo'))

        # スクレイピング準備
        self.wdc.settingScrape()

        # ページロード
        self.wdc.browserCtl.loadPage(lastNameInfoUrl)

        # pandasデータを返却する
        return self.wdc.browserCtl.createSearchResultDataFrame(**kwargs)

    class beautifulSoup(browserContainer.beautifulSoup):
        
        """
        ブラウザコンテナ:beautifulSoup
        """

        def __init__(self, _config: kojimaConfig):
            
            """
            コンストラクタ
            
            Parameters
            ----------
                _config : kojimaConfig
                    共通設定
            """

            super().__init__(_config)
            
            self.config = _config
            self.bc = gettr_lastNameList
            self.cbCreateSearchResultDataFrameByBeutifulSoup = self.createSearchResultDataFrameByBeutifulSoup
                            
        def createSearchResultDataFrameByBeutifulSoup(self, soup:BeautifulSoup, *args, **kwargs) -> DataFrame:
            
            """
            全国苗字ランキングをDataFrameで返す(By BeutifulSoup)
            
            Parameters
            ----------
            soup : BeautifulSoup
                BeautifulSoupオブジェクト
            
            """

            # スクレイピング結果から改行ｺｰﾄﾞを除去
            [tag.extract() for tag in soup(string='\n')]
            
            # class取得
            tables = soup.find_all(class_="simple")

            if tables:
                
                # 全国苗字情報model用意            
                lastNameInfo = recset(trn_last_name)
                
                for index in range(len(tables)):
                    try:
                        last_name_list = tables[index].find_all(class_="odd")
                        if len(last_name_list) > 5 and tables[index].attrs['width'] == '98%':
                            for index in range(len(last_name_list)):
                                
                                last_name_row = last_name_list[index].find_all('td')
                                # 順位
                                ranking = last_name_row[0].text.replace('位','')
                                # 苗字
                                last_name = last_name_row[1].text
                                # 人数
                                numPeple = last_name_row[2].text.replace('およそ','').replace('人','').replace(',','')
                                
                                # Modelに追加
                                lastNameInfoRow:trn_last_name = lastNameInfo.newRow()
                                lastNameInfoRow.ranking = ranking
                                lastNameInfoRow.last_name = last_name
                                lastNameInfoRow.numPeple = numPeple
                                lastNameInfo.addRow(lastNameInfoRow)

                                # コンソール出力
                                print('順位={0}位'.format(ranking))
                                print('苗字={0}'.format(last_name))
                        
                    except Exception as e: # その他例外
                        Logger.logging.error('Other Exception')
                        Logger.logging.error(str(e))
                
                return lastNameInfo.getDataFrame()
        



import os
import random
from LibHanger.Library.uwImport import JsonImporter
from LibHanger.Library.uwImage import uwImage
from pandas import DataFrame
from parroter.Library.parroterBase import parroterBase
from parroter.Library.kojima.kojimaConfig import kojimaConfig
from parroter.Library.kojima.kojimaGlobals import *
from parroter.Library.kojima.Getter.lastNameList import gettr_lastNameList

class kojimaBot(parroterBase):
    
    """
    児嶋Botクラス
    """
            
    def __init__(self, rootPath, lang:str = parroterBase.botLang.jp) -> None:
        
        """
        コンストラクタ

        Parameters
        ----------
        rootPath : str
            ルートパス
        lang : str
            言語
        """
        
        # 基底側コンストラクタ
        super().__init__(rootPath, lang)
                
        # gvに設定情報をセット
        gv.config = self.config
        gv.kojimaConfig = self.settingConfig(rootPath)
        
        # kojima共通設定をメンバ変数にセット
        self.parroterConfig = gv.kojimaConfig
        
        # キートークン設定
        self.setKeyToken()

        # ログ出力先設定
        self.settingLogOutputPath()
        
    def appName(self):
        
        """
        アプリ名
        """

        return 'kojima'
    
    def settingConfig(self, rootPath):
        
        """
        parroter共通設定
        
        Parameters
        ----------
        rootPath : str
            ルートパス
        """
        
        # kojima.ini
        kc = kojimaConfig()
        kc.getConfig(rootPath, os.path.join(self.config.startupCfg.configFolderPath, self.appName()))

        return kc
    
    def getLastName(self, *args, **kwargs):
        
        """
        苗字リストを取得してDataFrameとして返す
        """
        
        getter = gettr_lastNameList()
        return getter.getAllData(**kwargs)
    
    def getHelloTweetText(self):
        
        """
        挨拶文を取得する
        """
        
        # 挨拶文の定型テキスト文取得
        helloTemplateText = self.getHelloTweetTemplateText()
                
        return helloTemplateText
    
    def getHelloTweetImage(self):
        
        """
        名言ツイート画像を生成する
        """
        
        # 名言ツイート画像を生成する
        return self.generateTweetImage()
    
    def getHelloTweetTemplateText(self):

        """
        挨拶定型文を取得する
        """
        
        with open(os.path.join(self.rootPath, 
                               self.parroterConfig.appListDir, 
                               self.parroterConfig.appHelloTextFileName),'r',encoding='utf-8') as f:
            helloTemplateText= f.read()
        
        return helloTemplateText
            
    def getWiseSayingTweetData(self):
        
        """
        名言ツイート文を取得する
        """
                
        # 名言ツイートjsonファイルをDataFrameに変換
        ji = JsonImporter(self.rootPathFull)
        dfWiseSaying = ji.convertToDataFrame(gv.kojimaConfig.wiseSayingJsonDir, gv.kojimaConfig.wiseSayingJsonFileName)
        targetTweetNo = []
        if len(dfWiseSaying) > 0:
            targetTweetNo = range(1, len(dfWiseSaying) + 1)
        else:
            return []

        # 名言ツイートNoログ取得
        wiseSayingTweetNoLogFilePath = os.path.join(self.rootPath, 
                                                    gv.kojimaConfig.wiseSayingTweetNoLogDir, 
                                                    gv.kojimaConfig.wiseSayingTweetNoLogFileName)
        
        # 名言ツイートNoログ出力先ディレクトリ作成
        wiseSayingTweetNoLogFolderDir = os.path.dirname(wiseSayingTweetNoLogFilePath)
        if not os.path.exists(wiseSayingTweetNoLogFolderDir):
            os.makedirs(wiseSayingTweetNoLogFolderDir, exist_ok=True)
            
        wiseSayingTweetNoLog = []
        if os.path.exists(wiseSayingTweetNoLogFilePath):
            
            # 名言ツイートログ読込
            with open(wiseSayingTweetNoLogFilePath,'r',encoding='utf-8') as f:
                wiseSayingTweetNoLog = f.read().split('\n')
        
        # 対象ツイートNo絞り込み   
        if len(wiseSayingTweetNoLog) > 0:
            
            # ツイート済NoはtargetTweetNoから除外する
            targetTweetNo = [tweetNo for tweetNo in targetTweetNo if not tweetNo in wiseSayingTweetNoLog]

            # 対象ツイートNoが取得できなかった場合は名言ツイートログファイルを初期化する
            if len(targetTweetNo) == 0:
                
                with open(wiseSayingTweetNoLogFilePath, mode='w') as f:
                    targetTweetNo = range(1, len(dfWiseSaying) + 1)
                    
        # ツイートNoをランダム選択
        tweetNo = random.choice(targetTweetNo)

        # ツイートNoを名言ツイートログに書込
        with open(wiseSayingTweetNoLogFilePath, mode='a') as f:
            f.write(str(tweetNo))
            f.write('\n')
        
        # dfWiseSayingから対象となるツイートを取得する
        return dfWiseSaying[dfWiseSaying['no'] == tweetNo]
    
    def getWiseSayingPtrnData(self, wiseSayingTweetPattern):
        
        """
        名言ツイートパターンリストを取得する
        
        Parameters
        ----------
        wiseSayingTweetPattern : str
            名言ツイートパターンNo

        """
        
        # 名言ツイートパターンリストjsonファイルをDataFrameに変換
        ji = JsonImporter(self.rootPathFull)
        dfWiseSayingPtrn = ji.convertToDataFrame(gv.kojimaConfig.wiseSayingJsonDir, gv.kojimaConfig.wiseSayingPtrnJsonFileName)
        
        # 該当する名言パターンを取得
        srWiseSayingPtrn = dfWiseSayingPtrn[dfWiseSayingPtrn['pattern'] == wiseSayingTweetPattern].iloc[0]
        
        # 名言パターンリストクラスへ値セット
        gv.kojimaConfig.wiseSayingPattern.pattern = wiseSayingTweetPattern
        gv.kojimaConfig.wiseSayingPattern.font_name = srWiseSayingPtrn['font_name']
        gv.kojimaConfig.wiseSayingPattern.say_ipX = srWiseSayingPtrn['say{}_ipX'.format(self._lang)]
        gv.kojimaConfig.wiseSayingPattern.say_ipY = srWiseSayingPtrn['say{}_ipY'.format(self._lang)]
        gv.kojimaConfig.wiseSayingPattern.say_fontSize = srWiseSayingPtrn['say{}_fontSize'.format(self._lang)]
        gv.kojimaConfig.wiseSayingPattern.say_indentYPoint = srWiseSayingPtrn['say{}_indentYPoint'.format(self._lang)]
        gv.kojimaConfig.wiseSayingPattern.say_txtHeight = srWiseSayingPtrn['say{}_txtHeight'.format(self._lang)]
        gv.kojimaConfig.wiseSayingPattern.category_ipX = srWiseSayingPtrn['category{}_ipX'.format(self._lang)]
        gv.kojimaConfig.wiseSayingPattern.category_ipY = srWiseSayingPtrn['category{}_ipY'.format(self._lang)]
        gv.kojimaConfig.wiseSayingPattern.category_fontSize = srWiseSayingPtrn['category{}_fontSize'.format(self._lang)]
        gv.kojimaConfig.wiseSayingPattern.category_indentYPoint = srWiseSayingPtrn['category{}_indentYPoint'.format(self._lang)]
        gv.kojimaConfig.wiseSayingPattern.category_txtHeight = srWiseSayingPtrn['category{}_txtHeight'.format(self._lang)]
        gv.kojimaConfig.wiseSayingPattern.speaker_ipX = srWiseSayingPtrn['speaker{}_ipX'.format(self._lang)]
        gv.kojimaConfig.wiseSayingPattern.speaker_ipY = srWiseSayingPtrn['speaker{}_ipY'.format(self._lang)]
        gv.kojimaConfig.wiseSayingPattern.speaker_fontSize = srWiseSayingPtrn['speaker{}_fontSize'.format(self._lang)]
        gv.kojimaConfig.wiseSayingPattern.speaker_indentYPoint = srWiseSayingPtrn['speaker{}_indentYPoint'.format(self._lang)]
        gv.kojimaConfig.wiseSayingPattern.speaker_txtHeight = srWiseSayingPtrn['speaker{}_txtHeight'.format(self._lang)]
        gv.kojimaConfig.wiseSayingPattern.category_label = '出典:{}' if self._lang == self.botLang.jp else 'source:{}'

    def generateTweetImage(self):
        
        """
        名言ツイート画像を生成する
        """
        
        # 名言ツイートDataFrame取得
        dfWiseSaying = self.getWiseSayingTweetData()        
        if len(dfWiseSaying):
            srWiseSayingTweet = dfWiseSaying.iloc[0]
        else:
            return ''

        # 名言ツイートパターンリストDataFrame取得
        self.getWiseSayingPtrnData(srWiseSayingTweet['pattern'])
        wsp = gv.kojimaConfig.wiseSayingPattern
        
        # イメージクラスインスタンス
        imageFileName = srWiseSayingTweet['file_name']
        orgFolderDir = gv.kojimaConfig.orgImageFolderDir
        genFolderDir = gv.kojimaConfig.genImageFolderDir
        uwi = uwImage(self.rootPathFull, imageFileName, orgFolderDir, genFolderDir)

        # フォントファイル名取得
        fontFileName = wsp.font_name
        fontFilePath = os.path.join(gv.kojimaConfig.fontFolderDir, fontFileName)

        # テキストの文字数に応じて挿入開始位置を調整
        adjInsertPointY = self.getAdjustInsertPointY(srWiseSayingTweet['wise_saying_{}'.format(self._lang)], 
                                                     wsp.say_indentYPoint, 
                                                     wsp.say_txtHeight, 
                                                     4)
        
        # テキスト挿入(本文)
        uwi.setFont(fontFilePath, wsp.say_fontSize)
        uwi.insertText(srWiseSayingTweet['wise_saying_{}'.format(self._lang)], 
                       insertPoint=(wsp.say_ipX,adjInsertPointY + wsp.say_ipY), 
                       indentationYPoint=wsp.say_indentYPoint, 
                       textHeight=wsp.say_txtHeight)

        # テキスト挿入(category)
        if srWiseSayingTweet['category_{}'.format(self._lang)] != '':
            uwi.setFont(fontFilePath, wsp.category_fontSize)
            uwi.insertText(wsp.category_label.format(srWiseSayingTweet['category_{}'.format(self._lang)]), 
                           insertPoint=(wsp.category_ipX,wsp.category_ipY), 
                           indentationYPoint=wsp.category_indentYPoint, 
                           textHeight=wsp.category_txtHeight)

        # テキスト挿入(speaker)
        if srWiseSayingTweet['speaker_{}'.format(self._lang)] != '':
            uwi.setFont(fontFilePath, wsp.speaker_fontSize)
            uwi.insertText('@{}'.format(srWiseSayingTweet['speaker_{}'.format(self._lang)]), 
                           insertPoint=(wsp.speaker_ipX,wsp.speaker_ipY), 
                           indentationYPoint=wsp.speaker_indentYPoint, 
                           textHeight=wsp.speaker_txtHeight)

        # イメージを保存する
        return uwi.saveImage()
    
    def getReplyTweetImageText(self, replyFromTweet_text):
        
        """
        返信画像に挿入する返信ツイート内容を取得

        replyFromTweet_text : str
            返信元ツイート
        """
        
        # 全国苗字データDataFrame取得
        dfLastName = self.getLastNameData()
        
        # 返信ツイートDataFrame取得
        dfReplyTweet = self.getReplyTweetData()
                
        # 返信ツイートテキストに苗字が含まれるか確認
        srLastName = self.matchingLastName(dfLastName, replyFromTweet_text)
        
        # 取得した苗字を返信ツイートパターンと照合する
        srReplyTweet = self.matchingReplyTweetTextInLastName(replyFromTweet_text, srLastName, dfReplyTweet)
        
        # 返信ツイート内容を返す
        return srReplyTweet
    
    def getLastNameData(self):
        
        """
        全国苗字データ取得
        """        

        # 基底側で取得
        dfLastName = super().getLastNameData()
        # 文字数の降順にソートする
        dfLastName['order'] = dfLastName['last_name'].apply(lambda x: len(x))
        dfLastName = dfLastName.sort_values('order', ascending=False)

        # 戻り値を返す
        return dfLastName
    
    def matchingLastName(self, dfLastName:DataFrame, replyFromTweet_text):
        
        """
        返信ツイートテキストに苗字が含まれるか確認
        マッチングされた苗字データをSeriesで返す
        
        dfLastName : DataFrame
            全国苗字データDataFrame
        replyFromTweet_text : str
            返信元ツイート            
        """
        
        srMatchedLastName = None
        
        # 全国苗字データを1件ずつループ
        for index, row in dfLastName.iterrows():
            
            if row['last_name'] in replyFromTweet_text:
            
                # マッチした行をセットする
                srMatchedLastName = row
                break
                
        return srMatchedLastName
    
    def matchingReplyTweetTextInLastName(self, replyFromTweet_text, srLastName, dfReplyTweet:DataFrame):
        
        """
        取得した苗字を返信ツイートパターンと照合する
        マッチングされた返信ツイート内容をSeriesで返す

        replyFromTweet_text : str
            返信元ツイート
        srLastName : Series
            苗字データ
        dfReplyTweet : DataFrame
            返信ツイートデータ
        """
        
        # 返却用Series初期化
        srReplyTweet = None
        
        # ワイルドカード返信ツイート抽出
        dfWildCardPattern:DataFrame = dfReplyTweet[dfReplyTweet['keyword1'].str.contains('\*')]
        
        # ワイルドカード以外の返信ツイート抽出
        dfNotWildCardPattern:DataFrame = dfReplyTweet[dfReplyTweet['keyword1'].str.contains('\*') == False]
        
        # keyword1+keyword2
        for index, row in dfNotWildCardPattern.iterrows():
            
            # 苗字が返信元ツイートに含まれるか
            if row['keyword1'] + row['keyword2'] in replyFromTweet_text:
                srReplyTweet = row
                break
            else:
                continue

        # keyword1+keyword3
        if srReplyTweet is None:
            for index, row in dfNotWildCardPattern.iterrows():
                
                # 苗字が返信元ツイートに含まれるか
                if row['keyword1'] in replyFromTweet_text or (row['keyword3'] != '' and row['keyword3'] in replyFromTweet_text):
                    srReplyTweet = row
                    break
                else:
                    continue
        
        # マッチングしなかった場合はワイルドカードのいずれかとする
        if srReplyTweet is None:

            # 苗字判定用テキストファイル読込
            lastnameTargetStrTextFilePath = os.path.join(self.rootPath, 
                                                         gv.kojimaConfig.appListDir, 
                                                         gv.kojimaConfig.lastnameTargetStrTextFileName)
            with open(lastnameTargetStrTextFilePath,'r',encoding='utf-8') as f:
                targetStr = f.read().split('\n')
            
            # 特定の文字がマッチングされた苗字に含まれるか
            if not srLastName is None:
                matchedLastName = [matchStr for matchStr in targetStr if matchStr in srLastName['last_name']]
                if len(matchedLastName) > 0:
                    # 名前が惜しい
                    srReplyTweet = dfWildCardPattern[dfWildCardPattern['keyword1'] == '*1'].iloc[0]
                else:
                    # 名前が惜しくない
                    srReplyTweet = dfWildCardPattern[dfWildCardPattern['keyword1'] == '*3'].iloc[0]
            else:
                # 返信ツイートに名前が含まれない
                srReplyTweet = dfWildCardPattern[dfWildCardPattern['keyword1'] == '*2'].iloc[0]
        
        return srReplyTweet
    
    def generateReplyTweetImage(self, srReplyTweet):

        """
        返信画像を生成する

        replyTweetText : str
            返信ツイートテキスト
        """
        
        # 返信ツイートパターンDataFrame取得
        self.getReplyTweetPtrnData(srReplyTweet['pattern'])
        wsp = gv.kojimaConfig.replyTweetPattern
        
        # イメージクラスインスタンス
        imageFileName = srReplyTweet['file_name']
        orgFolderDir = gv.kojimaConfig.orgImageFolderDir
        genFolderDir = gv.kojimaConfig.genImageFolderDir
        uwi = uwImage(self.rootPathFull, imageFileName, orgFolderDir, genFolderDir)

        # フォントファイル名取得
        fontFileName = wsp.font_name
        fontFilePath = os.path.join(gv.kojimaConfig.fontFolderDir, fontFileName)

        # テキストの文字数に応じて挿入開始位置を調整
        adjInsertPointY = self.getAdjustInsertPointY(srReplyTweet['reply_text'], 
                                                     wsp.reply_indentYPoint, 
                                                     wsp.reply_txtHeight, 
                                                     4)
        
        # テキスト挿入(本文)
        uwi.setFont(fontFilePath, wsp.reply_fontSize)
        uwi.insertText(srReplyTweet['reply_text'], 
                       insertPoint=(wsp.reply_ipX,adjInsertPointY + wsp.reply_ipY), 
                       indentationYPoint=wsp.reply_indentYPoint, 
                       textHeight=wsp.reply_txtHeight,
                       fontColor=(0,0,0))

        # イメージを保存する
        return uwi.saveImage()



import LibHanger.Models.fields as fld
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.sql.elements import Null

# Baseクラス生成
Base = declarative_base()

class trn_last_name(Base):
	
	# テーブル名
	__tablename__ = 'last_name'
	
	# 列定義
	ranking = fld.NumericFields(6,0,primary_key=True,default=0)
	last_name = fld.CharFields(40,default='')
	numPeple = fld.NumericFields(9,0,default=0)



from Scrapinger.Library.scrapingConfig import scrapingConfig

class parroterConfig(scrapingConfig):
    
    """
    parotter共通設定クラス(parotterConfig)
    """ 
    
    class replyTweet_pattern():
        
        """
        返信ツイートパターンリスト
        """
        
        def __init__(self):
            
            """
            コンストラクタ
            """
            
            self.pattern = ''
            self.font_name = ''
            self.reply_ipX = 0
            self.reply_ipY = 0
            self.reply_fontSize = 0
            self.reply_indentYPoint = 0
            self.reply_txtHeight = 0
              
    def __init__(self):
        
        """
        コンストラクタ
        """
        
        # 基底側コンストラクタ
        super().__init__()

        self.consumer_key = ''
        """ コンシューマキー """

        self.consumer_secret = ''
        """ コンシューマキー(秘密) """
        
        self.access_token = ''
        """ アクセストークン """

        self.access_token_secret = ''
        """ アクセストークン(秘密) """

        self.hello_id_folder = ''
        """ 挨拶ツイートのIDログ格納先フォルダ """
        
        self.hello_id_filename = ''
        """ 挨拶ツイートのIDログファイル名 """
        
        self.reply_log_folder = ''
        """ 返信ツイートのIDログ格納先フォルダ """

        self.reply_log_filename = ''
        """ 返信ツイートのIDログファイル名 """
        
        self.appListDir = ''
        """ アプリリストフォルダパス """
        
        self.appHelloTextFileName = ''
        """ アプリ挨拶文テキストファイル名 """

        self.myoji_yurai_URL = 'https://myoji-yurai.net/prefectureRanking.htm?prefecture=%E5%85%A8%E5%9B%BD&page={}'
        """ myoji-yurai URL """

        self.lastNameJsonFileName = ''
        """ 全国苗字jsonファイル名 """

        self.lastNameJsonDir = ''
        """ 全国苗字jsonファイル格納先ディレクトリ """

        self.replyJsonFileName = ''
        """ 返信jsonファイル名 """

        self.replyPtrnJsonFileName = ''
        """ 返信パターンjsonファイル名 """

        self.replyJsonDir = ''
        """ 返信jsonファイル格納先ディレクトリ """
        
        self.orgImageFolderDir = ''
        """ オリジナルイメージ格納先フォルダ """

        self.genImageFolderDir = ''
        """ 生成イメージ格納先フォルダ """

        self.fontFolderDir = ''
        """ フォントファイル格納先フォルダ """
        
        self.replyTweetPattern = self.replyTweet_pattern()
        """ 返信ツイートパターンリスト """
        
        self.mentionsGetCount = 30
        """ 取得するメンションの最大数 """
        
    def getConfigFileName(self):
        
        """ 
        設定ファイル名 
        """

        return 'parroter.ini'
    
    def setInstanceMemberValues(self):
        
        """ 
        インスタンス変数に読み取った設定値をセットする
        """
        
        # 基底側実行
        super().setInstanceMemberValues()
        
        # consumer_key
        super().setConfigValue('consumer_key',self.config_ini,'KEYTOKEN','CONSUMER_KEY',str)

        # consumer_secret
        super().setConfigValue('consumer_secret',self.config_ini,'KEYTOKEN','CONSUMER_SECRET',str)

        # access_token
        super().setConfigValue('access_token',self.config_ini,'KEYTOKEN','ACCESS_TOKEN',str)

        # access_token_secret
        super().setConfigValue('access_token_secret',self.config_ini,'KEYTOKEN','ACCESS_TOKEN_SECRET',str)

        # hello-id Folder
        super().setConfigValue('hello_id_folder',self.config_ini,'LOG','HELLO_ID_FOLDER',str)

        # hello-id FileName
        super().setConfigValue('hello_id_filename',self.config_ini,'LOG','HELLO_ID_FILENAME',str)

        # reply-log Folder
        super().setConfigValue('reply_log_folder',self.config_ini,'LOG','REPLY_LOG_FOLDER',str)

        # reply-log FileName
        super().setConfigValue('reply_log_filename',self.config_ini,'LOG','REPLY_LOG_FILENAME',str)

        # App - ListDir
        super().setConfigValue('appListDir',self.config_ini,'DIR','APP_LIST_DIR',str)

        # App - helloText - FileName
        super().setConfigValue('appHelloTextFileName',self.config_ini,'FILE','APP_HELLO_TEXT_FILENAME',str)

        # myoji-yurai URL
        super().setConfigValue('myoji_yurai_URL',self.config_ini,'SITE','MYOJI_YURAI_URL',str)

        # lastName - filename
        super().setConfigValue('lastNameJsonFileName',self.config_ini,'FILE','LAST_NAME_JSON_FILENAME',str)

        # lastName - dir
        super().setConfigValue('lastNameJsonDir',self.config_ini,'DIR','LAST_NAME_JSON_DIR',str)

        # reply - filename
        super().setConfigValue('replyJsonFileName',self.config_ini,'FILE','REPLY_JSON_FILENAME',str)

        # reply_pattern - filename
        super().setConfigValue('replyPtrnJsonFileName',self.config_ini,'FILE','REPLY_PATTERN_JSON_FILENAME',str)

        # reply - dir
        super().setConfigValue('replyJsonDir',self.config_ini,'DIR','REPLY_JSON_DIR',str)

        # original image folder path
        super().setConfigValue('orgImageFolderDir',self.config_ini,'DIR','ORG_IMAGE_FOLDER_DIR',str)

        # generate image folder path
        super().setConfigValue('genImageFolderDir',self.config_ini,'DIR','GEN_IMAGE_FOLDER_DIR',str)

        # font file folder path
        super().setConfigValue('fontFolderDir',self.config_ini,'DIR','FONT_FOLDER_DIR',str)

        # mentions getcount
        super().setConfigValue('mentionsGetCount',self.config_ini,'DIR','MENTIONS_GET_COUNT',int)



import os
import sys
import math
import LibHanger.Library.uwLogger as Logger
from twython import Twython, TwythonError
from LibHanger.Library.uwImport import JsonImporter
from LibHanger.Library.uwConfig import cmnConfig
from parroter.Library.parroterConfig import parroterConfig

class parroterBase():
    
    """
    parroter基底クラス
    """
    
    class botLang():
        
        """
        言語設定
        """
        
        jp = 'Jp'
        """ 日本語 """

        en = 'En'
        """ 英語 """

    def __init__(self, rootPath, lang:str = botLang.jp) -> None:
        
        """
        コンストラクタ
        
        Parameters
        ----------
        rootPath : str
            ルートパス
        lang : str
            言語
        """

        # 言語設定
        self._lang = lang
        
        # LibHanger.ini
        dc = cmnConfig()
        dc.getConfig(rootPath)
        
        # ロガー設定
        Logger.setting(dc)

        # 共通設定
        self.config = dc
        self.parroterConfig:parroterConfig = None
        
        # キートークン初期化
        self.twitter = None
        
        # ルートパス
        self.rootPathFull = rootPath
        self.rootPath = os.path.dirname(rootPath)
    
    @property
    def appName(self):
        
        """
        アプリ名
        """

        return ''
    
    def settingLogOutputPath(self):
        
        """
        ログ出力先設定
        """
        
        # ログ出力先ディレクトリを作成
        self.hello_id_dir, self.reply_log_dir = self.createLogOutputDir()
        
        # ログ出力先取得
        self.hello_id_path, self.reply_log_path = self.getLogOutputPath(self.hello_id_dir, self.reply_log_dir)
    
    def settingConfig(self):
        
        """
        parroter共通設定(派生側でオーバーライドする)
        """
        
        pass
    
    def setKeyToken(self):
        
        """
        キートークンを設定する
        """
        
        # キートークン設定
        self.twitter = Twython(
                         self.parroterConfig.consumer_key
                        ,self.parroterConfig.consumer_secret
                        ,self.parroterConfig.access_token
                        ,self.parroterConfig.access_token_secret
                        )
    
    def tweetHello(self):
        
        """
        挨拶をツイートする
        """
        
        # 挨拶ツイート内容の設定
        helloTweetText = self.getHelloTweetText()

        # ツイートに含める画像の設定
        helloTweetImage = self.getHelloTweetImage()
                
        # 挨拶をツイートする
        response, result = self.tweet(helloTweetText, helloTweetImage)
        if result:
            
            try:
                                
                # 挨拶ツイートログ出力
                self.createHelloTweetLog(response, self.hello_id_path)
                                
            except Exception as e:                
                # エラーロギング
                Logger.logging.error(e.msg)
                # 処理を停止
                sys.exit(1)

        else:
            # 処理を停止
            sys.exit(1)
        
    def getHelloTweetText(self):
        
        """
        挨拶文を取得する
        """
        
        return ''
    
    def getHelloTweetImage(self):
        
        """
        挨拶ツイートに含める画像を取得する
        """
        
        return ''
    
    def getAdjustInsertPointY(self, insertText, indentYPoint, txtHeight, maxIndentCount):

        """
        テキスト挿入位置調整

        Parameters
        ----------
        insertText : str
            挿入文字列
        indentYPoint : int
            1行あたり文字数
        txtHeight : int
            文字高さ
        maxIndentCount : int
            最大字下げ数            
        """
                
        insertTextLen = len(insertText)
        if insertTextLen <= indentYPoint:
            adjInsertPointY = 0
        else:
            baseLength = indentYPoint * maxIndentCount
            indentCount = math.floor(baseLength / insertTextLen) if insertTextLen < baseLength else maxIndentCount
            adjInsertPointY = -1 * (txtHeight * indentCount)
        
        return adjInsertPointY
    
    def uploadImage(self, tweetImageFilePath):
        
        """
        イメージファイルをアップロードする

        Parameters
        ----------
        tweetImageFilePath : str
            イメージファイルパス        
        """
        
        # イメージファイルを開く
        image = open(tweetImageFilePath,'rb')
        # イメージファイルアップロード
        image_upl_inf = self.twitter.upload_media(media=image)
        # media_id取得
        media_id = image_upl_inf['media_id']                    

        # 戻り値を返す
        return media_id
    
    def tweet(self, tweetText, tweetImageFilePath = ''):
        
        """
        ツイートする
        
        Parameters
        ----------
        tweetText : str
            ツイート文
        tweetImageFilePath : str
            ツイート画像ファイルパス
        """
        
        response = None
        try:
            # 挨拶ツイート
            if tweetImageFilePath == '': # テキストのみ
                response = self.twitter.update_status(status=tweetText)
            else: # 画像付き
                if os.path.exists(tweetImageFilePath):
                    media_id = self.uploadImage(tweetImageFilePath)
                    response = self.twitter.update_status(status=tweetText, media_ids=[media_id])
                else:
                    response = self.twitter.update_status(status=tweetText)
        except TwythonError as e:
            # エラーロギング
            Logger.logging.error(e.msg)
            # Falseを返す
            return response, False
        else:
            return response, True
        
    def getLogOutputPath(self, hello_id_dir, reply_log_dir):
        
        """
        ログ出力先を返す

        Parameters
        ----------
        hello_id_dir : str
            挨拶ツイートIDログ出力先
        reply_log_dir : str
            返信ツイートIDログ出力先
        """
        
        # ログ出力先
        hello_id_path = os.path.join(hello_id_dir, 
                                     self.parroterConfig.hello_id_filename)
        reply_log_path = os.path.join(reply_log_dir, 
                                      self.parroterConfig.reply_log_filename)
        
        # ログ出力先を返す
        return hello_id_path, reply_log_path
    
    def createLogOutputDir(self):
        
        """
        ログ出力先ディレクトリ作成
        """

        # ログ出力先ディレクトリ
        hello_id_dir = os.path.join(self.rootPath, 
                                    self.parroterConfig.hello_id_folder)
        reply_log_dir = os.path.join(self.rootPath, 
                                     self.parroterConfig.reply_log_folder)
        
        # ログ出力先ディレクトリを作成
        os.makedirs(hello_id_dir, exist_ok=True)
        os.makedirs(reply_log_dir, exist_ok=True)
        
        # 作成したディレクトリパスを返す
        return hello_id_dir, reply_log_dir

    def createHelloTweetLog(self, response, hello_id_path):
        
        """
        挨拶ツイートログ出力

        Parameters
        ----------
        response : any
            tweet-response
        hello_id_path : str
            挨拶ツイートIDログ出力先        
        """
        
        # 挨拶ツイートログ出力
        with open(hello_id_path, mode='w') as f:
            f.write(response['id_str'])
            
    def outputReplyTweetLog(self, response, reply_log_path):
        
        """
        返信ツイートログ出力
        
        Parameters
        ----------
        response : any
            tweet-response
        reply_log_path : str
            返信ツイートIDログ出力先  
        """
        
        # 返信ツイートログ出力
        with open(reply_log_path, mode='a') as f:
            f.write(response['id_str'] + '\n')
    
    def doReply(self):
        
        """
        特定のツイートに対して返信する
        """
        
        # 返信するツイートID取得
        replyTargetTweetId = self.getHelloTweetLog(self.hello_id_path)
        
        # リプライカウンタ初期化
        replyCounter = 0
        
        for tweetId in replyTargetTweetId:
            
            # メンションを取得
            responses = self.getMentions()

            # フォロワーIDリストを取得
            follower_ids = self.getFollowerIdList()
            
            # 返信済リストを取得        
            replyTweetIdList = self.getReplyList()
            
            # 返信
            for response in responses:
                
                # フォロワー及び未返信ツイートかどうか判定
                if self.isReplyTweet(response, 
                                     follower_ids=follower_ids['ids'], 
                                     replyTweetIdList=replyTweetIdList):
                    
                    try:
                        # 返信をツイートする
                        replyResponse, result = self.replyTweet(response['id'], response['text'])
                        if result:

                            # 返信済リストにtweet_idを追記
                            self.outputReplyTweetLog(response, self.reply_log_path)

                            # リプライカウンタ++
                            replyCounter += 1
                            
                            # ログ出力
                            Logger.logging.info("reply complete. reply_tweet_id={}".format(replyResponse['id']))
                                                
                    except TwythonError as te:
                        Logger.logging.error(te.msg)
                        sys.exit(1)

            # ログ出力
            Logger.logging.info("doReply finished. reply Count={}".format(str(replyCounter)))
    
    @Logger.loggerDecorator("getting mentioins")
    def getMentions(self):
        
        """
        メンションを取得
        """
        
        responses = None
        try:
            responses = self.twitter.get_mentions_timeline(count=self.parroterConfig.mentionsGetCount)
        except TwythonError as e:
            Logger.logging.error(e.msg)
            sys.exit(1)
            
        return responses
    
    @Logger.loggerDecorator("getting follower id list")
    def getFollowerIdList(self):
        
        """
        フォロワーIDリストを取得
        """
    
        try:
            follower_ids = self.twitter.get_followers_ids(stringify_ids=True)
        except TwythonError as e:
            Logger.logging.error(e.msg)
            sys.exit(1)

        return follower_ids
    
    @Logger.loggerDecorator("getting reply id list")
    def getReplyList(self):

        """
        返信済リストを取得
        """        
        
        # 返信済リストを取得
        replyTweetIdList = []
        if os.path.exists(self.reply_log_path):
            
            with open(self.reply_log_path, mode='r', encoding='utf-8') as f:
                replyTweetIdList = f.read().split('\n')
                
        else:
            
            with open(self.reply_log_path, mode='w', encoding='utf-8') as f:
                pass
                
        return [replyTweetId for replyTweetId in replyTweetIdList if replyTweetId != '']
    
    def isReplyTweet(self, response, **kwargs):
        
        """
        返信対象ツイートか判定する

        Parameters
        ----------
        response : any
            tweet-response
        """
        
        # フォロワーリスト
        follower_ids = kwargs.get('follower_ids')
        # 返信済ツイートID
        replyTweetIdList = kwargs.get('replyTweetIdList')
        
        # 返信者のユーザーID取得
        usr_id = response['user']['id_str']
        
        # 返信対象かどうか判定結果を返す
        # フォロワーかつ返信済ツイートIDリストに含まれないツイートか
        return usr_id in follower_ids and not response['id_str'] in replyTweetIdList

    def getReplyTweetImageText(self, replyFromTweet_text):
        
        """
        返信画像に挿入するテキストを取得

        replyFromTweet_text : str
            返信されたツイートテキスト
        """
        
        pass
    
    def generateReplyTweetImage(self, replyTweetText):

        """
        返信画像を生成する

        replyTweetText : str
            返信ツイートテキスト
        """
        
        pass
    
    def replyTweet(self, replyTarget_tweet_id, replyFromTweet_text):

        """
        返信をツイートする

        replyTarget_tweet_id : str
            返信対象となるツイートID
        replyFromTweet_text : str
            返信されたツイートテキスト
        """

        replyResponse = None
        try:
            # 返信画像に挿入する返信ツイート内容を取得
            srReplyTweet = self.getReplyTweetImageText(replyFromTweet_text)

            # 返信画像を生成
            imageFilePath = self.generateReplyTweetImage(srReplyTweet)
            if imageFilePath:

                # 返信画像アップロード
                media_id = self.uploadImage(imageFilePath)
                
                # 返信をツイート
                replyResponse = self.twitter.update_status(status='',
                                                           in_reply_to_status_id=replyTarget_tweet_id, 
                                                           auto_populate_reply_metadata=True, 
                                                           media_ids=media_id)
            else:
                
                # 返信をツイート
                replyResponse = self.twitter.update_status(status=srReplyTweet['reply_text'], 
                                                           in_reply_to_status_id=replyTarget_tweet_id,
                                                           auto_populate_reply_metadata=True)

        except TwythonError as e:
            # エラーロギング
            Logger.logging.error(e.msg)
            # Falseを返す
            return replyResponse, False
        else:
            return replyResponse, True
                    
    def getHelloTweetLog(self, hello_id_path):
        
        """
        挨拶ツイートログ取得

        Parameters
        ----------
        hello_id_path : str
            挨拶ツイートIDログ出力先        
        """
        
        # 挨拶ツイートログ取得
        with open(hello_id_path, mode='r', encoding='utf-8') as f:
            return f.read().split('\n')
            
    def getLastNameData(self):
        
        """
        全国苗字データ取得
        """        
        
        # 全国苗字jsonファイルをDataFrameに変換
        ji = JsonImporter(self.rootPathFull)
        dfLastName = ji.convertToDataFrame(self.parroterConfig.lastNameJsonDir, self.parroterConfig.lastNameJsonFileName)

        return dfLastName
    
    def getReplyTweetData(self):
        
        """
        返信ツイートデータ取得
        """        
        
        # 返信ツイートjsonファイルをDataFrameに変換
        ji = JsonImporter(self.rootPathFull)
        dfReplyTweet = ji.convertToDataFrame(self.parroterConfig.replyJsonDir, self.parroterConfig.replyJsonFileName)

        return dfReplyTweet
        
    def getReplyTweetPtrnData(self, replyTweetPattern):
        
        """
        返信ツイートパターンデータ取得
        
        Parameters
        ----------
        replyTweetPattern : Series
            返信ツイートパターン        
        """        
        
        # 返信ツイートパターンjsonファイルをDataFrameに変換
        ji = JsonImporter(self.rootPathFull)
        dfReplyTweetPtrn = ji.convertToDataFrame(self.parroterConfig.replyJsonDir, self.parroterConfig.replyPtrnJsonFileName)

        # 該当する返信ツイートパターンを取得
        srReplyTweetPtrn = dfReplyTweetPtrn[dfReplyTweetPtrn['pattern'] == replyTweetPattern].iloc[0]

        # 返信ツイートパターンリストクラスへ値セット
        self.parroterConfig.replyTweetPattern.pattern = replyTweetPattern
        self.parroterConfig.replyTweetPattern.font_name = srReplyTweetPtrn['font_name']
        self.parroterConfig.replyTweetPattern.reply_ipX = srReplyTweetPtrn['reply{}_ipX'.format(self._lang)]
        self.parroterConfig.replyTweetPattern.reply_ipY = srReplyTweetPtrn['reply{}_ipY'.format(self._lang)]
        self.parroterConfig.replyTweetPattern.reply_fontSize = srReplyTweetPtrn['reply{}_fontSize'.format(self._lang)]
        self.parroterConfig.replyTweetPattern.reply_indentYPoint = srReplyTweetPtrn['reply{}_indentYPoint'.format(self._lang)]
        self.parroterConfig.replyTweetPattern.reply_txtHeight = srReplyTweetPtrn['reply{}_txtHeight'.format(self._lang)]



from setuptools import setup
from codecs import open
from os import path
import re

package_name = 'parroter'
subPackage_name1 = 'Library'
subPackage_name2 = 'Models'
subPackage_kojima = 'Library/kojima'
subPackage_kojima_Getter = 'Library/kojima/Getter'

root_dir = path.abspath(path.dirname(__file__))

def _requirements():
    return [name.rstrip() for name in open(path.join(root_dir, 'requirements.txt')).readlines()]


def _test_requirements():
    return [name.rstrip() for name in open(path.join(root_dir, 'test-requirements.txt')).readlines()]

with open(path.join(root_dir, package_name, '__init__.py')) as f:
    init_text = f.read()
    version = re.search(r'__version__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)
    license = re.search(r'__license__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)
    author = re. search(r'__author__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)
    author_email = re.search(r'__author_email__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)
    url = re.search(r'__url__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)

assert version
assert license
assert author
assert author_email
assert url

with open('README.md', encoding='utf-8') as f:
    long_description = f.read()


setup(
    name=package_name,
    packages=[
        package_name + '/' + subPackage_name1,
        package_name + '/' + subPackage_name2,
        package_name + '/' + subPackage_kojima,
        package_name + '/' + subPackage_kojima_Getter,
    ],

    version=version,

    license=license,

    install_requires=_requirements(),
    tests_require=_test_requirements(),

    author=author,
    author_email=author_email,

    url=url,

    description='twitter bot',
    long_description=long_description,
    keywords='twitter,bot',

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Database',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)


from parroter.Library.parroterGlobals import parroterGlobal
from parroter.Library.kojima.kojimaConfig import kojimaConfig

class kojimaGlobal(parroterGlobal):
    
    """
    児嶋botグローバル設定クラス
    """
    
    def __init__(self):
        
        """
        コンストラクタ
        """
        
        # 基底側コンストラクタ呼び出し
        super().__init__()
        
        self.kojimaConfig:kojimaConfig = None
        """ kojima共通設定 """

# インスタンス生成(import時に実行される)
gv = kojimaGlobal()